package magie.assert;

// imports
import java.util.Date;
import java.util.Hashtable;
import java.util.Enumeration;
import java.text.SimpleDateFormat;
import java.io.StringWriter;
import java.io.PrintWriter;

/**
 * Class Assert is used to create Asserts
 *
 * @author  Markus Giebeler
 * @version $Revision: 1.5 $
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.<p>
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.<p>
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA<br>
 * http://www.gnu.org/copyleft/lesser.html"
 */
public class Assert {
  // constants
  public static final int TYPE_SILENT    = 0;
  public static final int TYPE_CONSOLE   = 1;
  public static final int TYPE_DIALOG    = 2;
  public static final int TYPE_EXCEPTION = 3;
  public static final int TYPE_FILE      = 4;
  /** set the default Handler */
  public static final int DEFAULT_HANDLER = TYPE_SILENT;
	static final String LONG_DATE_FORMAT    = "dd.MM.yyy - HH:mm ";
  static final String DEFAULT_MESSAGE     = "Assertion Failed";
  static final String TIME_PROFILE        = "Time Profile";
  static final String CLIENT_DATA         = "Client Data";
  static final String ERROR_PROTOCOL      = "Error Protocol";
  private static Hashtable timer=new Hashtable();

  // static fields
  /** Field handler (IAssertManager) is used to handle the assert */
  private static IAssertManager handler;

  // attributes
  /** Field timestamp (Date) is used to hold Date/Time of the Assertion */
  private Date timestamp;
  /** Field description (String) is used to <Enter description> */
  private String description;
  /** Field stack (String) is used to <Enter description> */
  private String stack;

 /**
  * Construction
  */
  public Assert() {
    setTimestamp(new Date());
  }

 //-----------------------------------------------------------------------------
 // Assert methods
 //-----------------------------------------------------------------------------
 /**
  * Asserts that a condition is true. If it isn't, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  *
  * @param message (String) message describing assertion
  * @param condition (boolean) a condition; assert is failed if condition is false
  */
  public static void assert(String message, boolean condition) {
    if (!condition) {
	    Assert ass = new Assert();
	    ass.setDescription(message);
      StringWriter stringWriter = new StringWriter();
      (new Exception()).printStackTrace(new PrintWriter(stringWriter));
      ass.setStack(stringWriter.toString());
      getHandler().handle(ass);
	  }
  }

 /**
  * Asserts that a condition is true. If it isn't an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  * <br>Uses a default message to 'describe' the assert
  *
  * @param condition (boolean) a condition; assert is failed if condition is false
  */
  public static void assert(boolean condition) {
    assert(DEFAULT_MESSAGE, condition);
  }

 /**
  * Asserts that the arguments 'expected' and 'was' are equal.
  * If they are not, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  *
  * @param message (String) message describing assertion
  * @param expected (Object) the expected value for a certain state to be tested
  * @param was (Object) the real value for the state to be tested
  */
  public static void assertEquals(String message, Object expected, Object was) {
    String s = ": Expected: "+expected.toString()+" but was: "+was.toString()+".";
    assert(message+s, expected.equals(was));
  }

 /**
  * Asserts that the arguments 'expected' and 'was' are equal.
  * If they are not, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  * <br>Uses a default message to 'describe' the assert
  *
  * @param expected (Object) the expected value for a certain state to be tested
  * @param was (Object) the real value for the state to be tested
  */
  public static void assertEquals(Object expected, Object was) {
    assertEquals(DEFAULT_MESSAGE, expected, was);
  }

 /**
  * Asserts that the arguments 'expected' and 'was' are equal.
  * If they are not, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  *
  * @param message (String) message describing assertion
  * @param expected (String) the expected value for a certain state to be tested
  * @param was (String) the real value for the state to be tested
  */
  public static void assertEquals(String message, String expected, String was) {
    String s = ": Expected: \""+expected+"\" but was: \""+was+"\".";
    assert(message+s, expected.equals(was));
  }

 /**
  * Asserts that the arguments 'expected' and 'was' are equal.
  * If they are not, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  * <br>Uses a default message to 'describe' the assert
  *
  * @param expected (String) the expected value for a certain state to be tested
  * @param was (String) the real value for the state to be tested
  */
  public static void assertEquals(String expected, String was) {
    assertEquals(DEFAULT_MESSAGE, expected, was);
  }

 /**
  * Asserts that the arguments 'expected' and 'was' are equal.
  * If they are not, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  *
  * @param message (String) message describing assertion
  * @param expected (int) the expected value for a certain state to be tested
  * @param was (int) the real value for the state to be tested
  */
  public static void assertEquals(String message, int expected, int was) {
    String s = ": Expected: "+expected+" but was: "+was+".";
    assert(message+s, expected==was);
  }

 /**
  * Asserts that the arguments 'expected' and 'was' are equal.
  * If they are not, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  * <br>Uses a default message to 'describe' the assert
  *
  * @param expected (int) the expected value for a certain state to be tested
  * @param was (int) the real value for the state to be tested
  */
  public static void assertEquals(int expected, int was) {
    assertEquals(DEFAULT_MESSAGE, expected, was);
  }

 /**
  * Asserts that the arguments 'expected' and 'was' are equal.
  * If they are not, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  *
  * @param message (String) message describing assertion
  * @param expected (float) the expected value for a certain state to be tested
  * @param was (float) the real value for the state to be tested
  */
  public static void assertEquals(String message, float expected, float was, float delta) {
    String s = ": Expected: "+expected+" but was: "+was+".";
    assert(message+s, Math.abs(expected-was) > delta);
  }

 /**
  * Asserts that the arguments 'expected' and 'was' are equal.
  * If they are not, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  * <br>Uses a default message to 'describe' the assert
  *
  * @param expected (float) the expected value for a certain state to be tested
  * @param was (float) the real value for the state to be tested
  */
  public static void assertEquals(float expected, float was, float delta) {
    assertEquals(DEFAULT_MESSAGE, expected, was, delta);
  }

 /**
  * Asserts that the arguments 'expected' and 'was' are equal.
  * If they are not, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  *
  * @param message (String) message describing assertion
  * @param expected (double) the expected value for a certain state to be tested
  * @param was (double) the real value for the state to be tested
  */
  public static void assertEquals(String message, double expected, double was, double delta) {
    String s = ": Expected: "+expected+" but was: "+was+".";
    assert(message+s, Math.abs(expected-was) > delta);
  }

 /**
  * Asserts that the arguments 'expected' and 'was' are equal.
  * If they are not, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  * <br>Uses a default message to 'describe' the assert
  *
  * @param expected (double) the expected value for a certain state to be tested
  * @param was (double) the real value for the state to be tested
  */
  public static void assertEquals(double expected, double was, double delta) {
    assertEquals(DEFAULT_MESSAGE, expected, was, delta);
  }

 /**
  * Asserts that the arguments 'expected' and 'was' are equal.
  * If they are not, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  *
  * @param message (String) message describing assertion
  * @param expected (long) the expected value for a certain state to be tested
  * @param was (long) the real value for the state to be tested
  */
  public static void assertEquals(String message, long expected, long was) {
    String s = ": Expected: "+expected+" but was: "+was+".";
    assert(message+s, expected==was);
  }

 /**
  * Asserts that the arguments 'expected' and 'was' are equal.
  * If they are not, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  * <br>Uses a default message to 'describe' the assert
  *
  * @param expected (long) the expected value for a certain state to be tested
  * @param was (long) the real value for the state to be tested
  */
  public static void assertEquals(long expected, long was) {
    assertEquals(DEFAULT_MESSAGE, expected, was);
  }

 /**
  * Asserts that the argument 'was' is not null. If it isn't, an Assert
  * is instantiated and passed to the assert handler in order to handle it.
  *
  * @param message (String) message describing assertion
  * @param was (Object) the real value for the state to be tested
  */
  public static void assertNotNull(String message, Object was) {
    assert(message, was!=null);
  }

 /**
  * Asserts that the argument 'was' is not null. If it isn't, an Assert
  * is instantiated and passed to the assert handler in order to handle it.
  *
  * @param message (String) message describing assertion
  * @param was (Object) the real value for the state to be tested
  */
  public static void assertNotNull(Object was) {
    assertNotNull(DEFAULT_MESSAGE, was);
  }

 /**
  * Asserts that the argument 'was' is null. If it isn't, an Assert
  * is instantiated and passed to the assert handler in order to handle it.
  * <br>Uses a default message to 'describe' the assert
  *
  * @param was (Object) the real value for the state to be tested
  */
  public static void assertNull(String message, Object was) {
    if(was!=null) {
      String s = ": Expected: <null> but was: "+was.toString()+".";
      assert(message+s, was==null);
    }
  }

 /**
  * Asserts that the argument 'was' is null. If it isn't, an Assert
  * is instantiated and passed to the assert handler in order to handle it.
  * <br>Uses a default message to 'describe' the assert
  *
  * @param was (Object) the real value for the state to be tested
  */
  public static void assertNull(Object was) {
    assertNull(DEFAULT_MESSAGE, was);
  }

 /**
  * Asserts that the arguments 'expected' and 'was' are the same.
  * If they are not, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  *
  * @param message (String) message describing assertion
  * @param expected (Object) the expected value for a certain state to be tested
  * @param was (Object) the real value for the state to be tested
  */
  public static void assertSame(String message, Object expected, Object was) {
    assert(message, expected==was);
  }

 /**
  * Asserts that the arguments 'expected' and 'was' are the same.
  * If they are not, an Assert is instantiated and
  * passed to the assert handler in order to handle it.
  * <br>Uses a default message to 'describe' the assert
  *
  * @param expected (Object) the expected value for a certain state to be tested
  * @param was (Object) the real value for the state to be tested
  */
  public static void assertSame(Object expected, Object was) {
    assertEquals(DEFAULT_MESSAGE, expected, was);
  }

 /**
  * start the timer
  *
  * @param key (String) dictionary key for timer entries
  */
  public static void startTimer(String key) {
    getTimer().put(key, new TimerEntry());
  }

 /**
  * start the timer (uses default key)
  */
  public static void startTimer() {
    startTimer(TIME_PROFILE);
  }

 /**
  * stop the timer (uses default key)
  */
  public static void stopTimer(String key) {
    TimerEntry te = (TimerEntry) getTimer().get(key);
    te.setEndTime(System.currentTimeMillis());
  }

 /**
  * stop the timer (uses default key)
  */
  public static void stopTimer() {
    stopTimer(TIME_PROFILE);
  }

 /**
  * clear the timer
  */
  public static void clearTimer() {
    setTimer(new Hashtable());
  }

 /**
  * create and handle Assert based on timer values
  *
  * @param message (String) a message, e.g. referring the source of the origin of that method call
  */
  public static void assertTimer(String message) {
	  Assert ass = new Assert();
	  ass.setDescription(TIME_PROFILE+": "+message);
    StringWriter stringWriter = new StringWriter();
    Enumeration e=getTimer().keys();
    while(e.hasMoreElements()) {
      String key = (String) e.nextElement();
      TimerEntry te = (TimerEntry) getTimer().get(key);
      stringWriter.write(key);
      stringWriter.write("=");
      stringWriter.write(te.toString());
      stringWriter.write("\r\n");
    }
    ass.setStack(stringWriter.toString());
    getHandler().handle(ass);
  }

 //-----------------------------------------------------------------------------
 // String representation
 //-----------------------------------------------------------------------------
 /**
  * String representation of the Assert for simple output
  *
  * @return a String describing the Assert
  */
  public String toString() {
    SimpleDateFormat df = new SimpleDateFormat(LONG_DATE_FORMAT);
    String s=df.format(getTimestamp());
    if(getDescription()==null) {
      s += DEFAULT_MESSAGE;
    } else {
      s += getDescription();
    }
    return s;
  }

 /**
  * String representation of the Assert for advanced output; includes Stack
  *
  * @return a String describing the Assert
  */
  public String detailedString() {
    SimpleDateFormat df = new SimpleDateFormat(LONG_DATE_FORMAT);
    String s=df.format(getTimestamp());
    if(getDescription()==null) {
      s += DEFAULT_MESSAGE;
    } else {
      s += getDescription();
    }
    s += "\r\n"+getStack();
    return s;
  }

 //-----------------------------------------------------------------------------
 // Assert Handler Management
 //-----------------------------------------------------------------------------
 /**
  * Set handler to a DialogAssertManager
  */
  public static void dialog() {
    setHandler(new IAssertManager() {
      public void handle(Assert ass) {
        new DialogAssertManager(ass);
      }
    });
  }

 /**
  * Set handler to a silent AssertManager
  */
  public static void silent() {
    setHandler(new IAssertManager() {
      public void handle(Assert ass) {
      }
    });
  }

 /**
  * Set handler to a AssertManager writing to the Console
  */
  public static void console() {
    setHandler(new IAssertManager() {
      public void handle(Assert ass) {
        System.out.println(ass.toString());
      }
    });
  }

 /**
  * Set handler to a AssertManager throw Exception
  */
  public static void exception() {
    setHandler(new IAssertManager() {
      public void handle(Assert ass) {
        throw new RuntimeException(ass.getDescription());
      }
    });
  }

 /**
  * Set handler to a FileAssertManager
  */
  public static void file() {
    setHandler(new FileAssertManager());
  }

 /**
  * write some client data to the handler
  *
  * @param data (String) the data
  */
  public static void write(String data){
	  Assert ass = new Assert();
	  ass.setDescription(CLIENT_DATA);
    ass.setStack(data);
    getHandler().handle(ass);
  }

 /**
  * write some client data to the handler
  *
  * @param data (String) the data
  */
  public static void write(Exception e) {
    Assert ass = new Assert();
	  ass.setDescription(ERROR_PROTOCOL);
    StringWriter stringWriter = new StringWriter();
    e.printStackTrace(new PrintWriter(stringWriter));
    ass.setStack(stringWriter.toString());
    getHandler().handle(ass);
  }

 /**
  * write some client data to the handler
  *
  * @param data (String) the data
  */
  public static void write(Exception e, String message) {
    Assert ass = new Assert();
	  ass.setDescription(ERROR_PROTOCOL+": "+message);
    StringWriter stringWriter = new StringWriter();
    e.printStackTrace(new PrintWriter(stringWriter));
    ass.setStack(stringWriter.toString());
    getHandler().handle(ass);
  }


 //-----------------------------------------------------------------------------
 // Static Accessing
 //-----------------------------------------------------------------------------
 /**
  * Set contents of field handler to the given value
  *
  * @param newValue (in) new value (IAssertManager) for field handler
  */
  public final static void setHandler(IAssertManager newValue) {
    handler = newValue;
  }

 /**
  * Set assert handler depending on the given type
  *
  * @param type (in) type of the reqired handler
  */
  public static void setHandler(int type) {
    switch (type) {
	    case TYPE_SILENT: {
        silent();
	  	  break;
      }
	    case TYPE_CONSOLE: {
        console();
	  	  break;
      }
	    case TYPE_EXCEPTION: {
        exception();
	  	  break;
      }
	    case TYPE_DIALOG: {
        dialog();
	  	  break;
      }
	    case TYPE_FILE: {
        file();
	  	  break;
      }
	    default: {
        silent();
		  }
    }
  }

 /**
  * Return contents of field handler
  *
  * @return (out) contents (IAssertManager) of field handler
  */
  public final static IAssertManager getHandler() {
    if(handler==null) {
      setHandler(DEFAULT_HANDLER);
    }
    return handler;
  }

 //-----------------------------------------------------------------------------
 // Accessing
 //-----------------------------------------------------------------------------
 /**
  * Return contents of field description
  *
  * @return (out) contents (String) of field description
  */
  public final String getDescription() {
    return description;
  }

 /**
  * Set contents of field description to the given value
  *
  * @param newValue (in) new value (String) for field description
  */
  public final void setDescription(String newValue) {
    this.description = newValue;
  }

 /**
  * Return contents of field timestamp
  *
  * @return (out) contents (Date) of field timestamp
  */
  public final Date getTimestamp() {
    return timestamp;
  }

 /**
  * Set contents of field timestamp to the given value
  *
  * @param newValue (in) new value (Date) for field timestamp
  */
  public final void setTimestamp(Date newValue) {
    this.timestamp = newValue;
  }

 /**
  * Return contents of field stack
  *
  * @return (out) contents (String) of field stack
  */
  public final String getStack() {
    return stack;
  }

 /**
  * Set contents of field stack to the given value
  *
  * @param newValue (in) new value (String) for field stack
  */
  public final void setStack(String newValue) {
    this.stack = newValue;
  }

 /**
  * Return contents of field timer
  *
  * @return (out) contents (Hashtable) of field timer
  */
  public static final Hashtable getTimer() {
    if(timer==null) {
      clearTimer();
    }
    return timer;
  }

 /**
  * Set contents of field timer to the given value
  * 
  * @param newValue (in) new value (Hashtable) for field timer
  */
  public static final void setTimer(Hashtable newValue) {
    timer=newValue;
  }
} // end of class Assert

