package magie.assert;

// imports
import java.awt.*;
import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

/**
 * Class DialogAssertManager is used to display a failed Assertion in a dialog
 * 
 * @author  Markus Giebeler
 * @version $Revision: 1.4 $
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.<p>
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.<p>
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA<br>
 * http://www.gnu.org/copyleft/lesser.html"
 */
public class DialogAssertManager extends JDialog {
  private JPanel panel = new JPanel();
  private JLabel dLabel = new JLabel();
  private JButton okButton = new JButton();
  private JScrollPane textScroller = new JScrollPane();
  private JLabel failedLabel = new JLabel();
  private JLabel stackLabel = new JLabel();
  private JScrollPane stackScroller = new JScrollPane();
  private JTextArea stack = new JTextArea();
  private JEditorPane description = new JEditorPane();
  private JButton saveButton = new JButton();

  /** field assert contains the Assert */
  private Assert assert;

 /**
  * Construction
  */
  public DialogAssertManager(Frame frame, String title, boolean modal, Assert ass) {
    super(frame, title, modal);
    try  {
      // init widgets
      jbInit();

      // save Assert and set fields
      setAssert(ass);
      description.setText(getDescription());
      stack.setText(getStack());

  	  // pack and display window
      pack();
    	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    	Dimension frameSize = this.getSize();
    	if (frameSize.height > screenSize.height)
    	  frameSize.height = screenSize.height;
    	if (frameSize.width > screenSize.width)
    	  frameSize.width = screenSize.width;
    	this.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
      this.setVisible(true);
    }
    catch(Exception ex) {
      ex.printStackTrace();
    }
  }
 /**
  * Construction (variant)
  */
  public DialogAssertManager(Assert ass) {
    this(null, "Assert Manager", true, ass);
  }

 /**
  * JBuilder generated initialization code
  */
  private void jbInit() throws Exception {
    panel.setLayout(null);
    panel.setPreferredSize(new Dimension(500, 400));
    dLabel.setText("Description:");
    dLabel.setBounds(new Rectangle(15, 42, 88, 23));
    okButton.setFocusPainted(false);
    okButton.setText(" OK ");
    okButton.setBounds(new Rectangle(14, 349, 79, 27));
    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        close();
      }
    });
    textScroller.setAutoscrolls(true);
    textScroller.setBounds(new Rectangle(15, 71, 470, 81));
    failedLabel.setFont(new Font("Dialog", 1, 18));
    failedLabel.setForeground(Color.red);
    failedLabel.setText("An Assertion Failed!!!");
    failedLabel.setBounds(new Rectangle(15, 4, 353, 36));
    stackLabel.setText("Stack:");
    stackLabel.setBounds(new Rectangle(15, 161, 82, 17));
    stackScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    stackScroller.setAutoscrolls(true);
    stackScroller.setBounds(new Rectangle(15, 186, 470, 147));
    stack.setBackground(Color.lightGray);
    stack.setSelectionColor(SystemColor.textHighlight);
    stack.setSelectedTextColor(SystemColor.textHighlightText);
    stack.setFont(new java.awt.Font("Dialog", 0, 11));
    description.setBackground(Color.lightGray);
    description.setSelectionColor(SystemColor.textHighlight);
    description.setEditable(false);
    description.setSelectedTextColor(SystemColor.textHighlightText);
    description.setFont(new java.awt.Font("Dialog", 0, 12));
    saveButton.setText("Save Assert");
    saveButton.setBounds(new Rectangle(104, 349, 144, 27));
    saveButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        saveAssert();
      }
    });
    getContentPane().add(panel);
    panel.add(textScroller, null);
    textScroller.getViewport().add(description, null);
    panel.add(dLabel, null);
    panel.add(failedLabel, null);
    panel.add(stackLabel, null);
    panel.add(stackScroller, null);
    panel.add(okButton, null);
    panel.add(saveButton, null);
    stackScroller.getViewport().add(stack, null);
  }

 /**
  * close Dialog after ok action
  */
  private void close() {
    dispose();
  }

  // static initializer for setting look & feel
  static {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }
    catch (Exception e) {}
  }

 /**
  * replace String a by String b in String s
  *
  * @param s (String) the String to be processed
  * @param a (String) substring to be replaced
  * @param b (String) String which shall replace String a
  * @return (String) the result String
  */
  public static String replace(String s, String a, String b) {
    if(s==null) return "";
    String t=s;
    int i=-1;
    while((i=t.indexOf(a, i+1))!=-1) {
      t=t.substring(0,i)+b+t.substring(i+a.length());
      i += a.length();
    }
    return t;
  }

 /**
  * set the assert
  */
  public void setAssert(Assert ass) {
    this.assert=ass;
  }

 /**
  * return the assert
  */
  public Assert getAssert() {
    return assert;
  }

 /**
  * return the (processed) assert stack
  */
  public String getStack() {
    String s=getAssert().getStack();
    s=replace(s,"java.lang.Exception\r\n","");
    s=replace(s,"\tat ","");
    return s;
  }

 /**
  * return the (processed) assert description
  */
  public String getDescription() {
    String s=getAssert().getDescription();
    s=replace(s,"Expected:","\r\nExpected:");
    return s;
  }

 /**
  * save assert to a file and open it with Notepad
  */
  private void saveAssert() {
    FileAssertManager handler=new FileAssertManager();
    handler.handle(getAssert());
    String f=handler.getFilename();
    try {
      Runtime.getRuntime().exec("notepad \""+f+"\"");
    } catch(IOException ex) {
    }
    close();
  }
} // end of class DialogAssertManager

