package magie.assert;

// imports
import java.util.*;
import java.text.SimpleDateFormat;
import java.io.*;

/**
 * Class FileAssertManager is used to write Assert data into a file
 * 
 * @author  Markus Giebeler
 * @version $Revision: 1.6 $ 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.<p>
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.<p>
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA<br>
 * http://www.gnu.org/copyleft/lesser.html"
 */
public class FileAssertManager implements IAssertManager {
  // constants
  static final String PACKAGE_NAME       = "magie\\assert";
	static final String PROTOCOL_FILENAME  = "Assert-";
	static final String PROTOCOL_EXTENSION = ".txt";
	static final String DATE_FORMAT = "yyMMdd";
	static final String LONG_DATE_FORMAT = "dd.MM.yyy - HH:mm";
  static final String LINE = "\r\n---------------------------------------------------------------------";
  // attributes
  private String filename;

 /**
  * replace String a by String b in String s
  *
  * @param s (String) the String to be processed
  * @param a (String) substring to be replaced
  * @param b (String) String which shall replace String a
  * @return (String) the result String
  */
  public static String replace(String s, String a, String b) {
    if(s.indexOf(a)==-1) return s;
    int last = 0;
    int size = a.length();
    int i = -size;

    // scan for occurences
    Vector list = new Vector();
    while((i=s.indexOf(a, i + size))!=-1) {
      list.addElement(new Integer(i));
    }

    // writing substrings to stream
    StringWriter writer = new StringWriter();
    for(i=0; i<list.size(); i++) {
      int index = ((Integer)list.elementAt(i)).intValue();
      if(index >= last) {
        writer.write(s.substring(last,index));
        writer.write(b);
        last = index + size;
      }
    }
    writer.write(s.substring(last, s.length()));
    return writer.toString();
  }

 /**
  * handles a given assert
  *
  * @param ass (Assert) the Assert
  */
  public void handle(Assert ass) {
    SimpleDateFormat df = new SimpleDateFormat(DATE_FORMAT);
    String f = PROTOCOL_FILENAME+df.format(new Date())+PROTOCOL_EXTENSION;
    File file=new File(f);
    filename=file.getAbsolutePath().toString();

    // try to read a the assert file
    StringWriter contents=new StringWriter();
    BufferedReader reader;
    try {
      reader = new BufferedReader(new FileReader(filename));
      while(reader.ready()) {
        int c = reader.read();
        contents.write(c);
      }
      reader.close();
    } catch(FileNotFoundException e) {
      // if not possible to read file, create a new file
      df = new SimpleDateFormat(LONG_DATE_FORMAT);
      contents.write("ASSERT FILE created: "+df.format(new Date()));
    } catch(IOException io) {
    }
    // append text of current assert
    String s = contents.toString()+LINE+"\r\n"+ass.detailedString();
    s=replace(s, "\n", "\r\n");
    s=replace(s, "\r\r", "\r");

    // write file
    BufferedWriter writer;
    try {
      writer = new BufferedWriter(new FileWriter(filename));
      writer.write(s);
      writer.close();
    } catch(IOException e) {
    }
  }

 /**
  * Return contents of field filename
  * 
  * @return (String) contents of field filename
  */
  public final String getFilename() {
    return filename;
  }

 /**
  * Set contents of field filename to the given value
  * 
  * @param newValue (String) new value for field filename
  */
  public final void setFilename(String newValue) {
    this.filename = newValue;
  }

} // end of class FileAssertManager

