package magie.assert;

// imports
import java.text.DecimalFormat;

/**
 * Class TimerEntry is used by the class Assert to create Timer records
 * 
 * @author  Markus Giebeler
 * @version $Revision: 1.4 $ 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.<p>
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.<p>
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA<br>
 * http://www.gnu.org/copyleft/lesser.html"
 */
public class TimerEntry {
  private static final double THOUSAND = 1000d;
  /** Field startTime (long) is used to save start time */
  private long startTime;
  /** Field endTime (long) is used to save end time */
  private long endTime;

  public TimerEntry() {
    this.setStartTime(System.currentTimeMillis());
    this.setEndTime(0);
  }

 /**
  * provide a String presentation of myself
  */
  public String toString() {
    double runTime;
    String additionalString = " sec.";
    DecimalFormat nf = new DecimalFormat("####0.000");
    if(getEndTime()<getStartTime()) {
		  runTime = (System.currentTimeMillis() - startTime) / THOUSAND;
      additionalString += " (Running)";
    } else {
		  runTime = (endTime - startTime) / THOUSAND;
    }
    return nf.format(runTime)+additionalString;
  }

 /**
  * Return contents of field startTime
  *
  * @return (out) contents (long) of field startTime
  */
  public final long getStartTime() {
    return startTime;
  }

 /**
  * Set contents of field startTime to the given value
  * 
  * @param newValue (in) new value (long) for field startTime
  */
  public final void setStartTime(long newValue) {
    this.startTime = newValue;
  }

 /**
  * Return contents of field endTime
  * 
  * @return (out) contents (long) of field endTime
  */
  public final long getEndTime() {
    return endTime;
  }

 /**
  * Set contents of field endTime to the given value
  * 
  * @param newValue (in) new value (long) for field endTime
  */
  public final void setEndTime(long newValue) {
    this.endTime = newValue;
  }



} // end of class TimerEntry

